# Copyright (c) 2019-2025,
# Lawrence Livermore National Security, LLC;
# See the top-level NOTICE for additional details. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

import units_llnl as u
import math
import copy


def test_basic_measurement():
    m1 = u.Measurement("10 m")
    m2 = u.Measurement("2.5 s")
    m3 = m1 / m2
    m4 = u.Measurement("4.0 m/s")
    assert m3 == m4


def test_Quantity_alias():
    m1 = u.Quantity("10 m")
    m2 = u.Quantity("2.5 s")
    m3 = m1 / m2
    m4 = u.Quantity("4.0 m/s")
    assert m3 == m4


def test_basic_measurement2():
    m1 = u.Measurement(10, "m")
    m2 = u.Measurement(2.5, "s")
    m3 = m1 / m2
    m4 = u.Measurement(4.0, "m/s")
    assert m3 == m4


def test_basic_measurement3():

    u1 = u.Unit("m")
    u2 = u.Unit("s")

    m1 = u.Measurement(10, u1)
    m2 = u.Measurement(2.5, u2)

    m4 = u.Measurement(4.0, u1 / u2)
    assert m1 / m2 == m4
    assert m4.units == u1 / u2


def test_basic_measurement3():

    u1 = u.Unit("m")
    u2 = u.Unit("s")

    m1 = u.Measurement(value=10, unit=u1)
    m2 = u.Measurement(unit=u2, value=2.5)

    m4 = u.Measurement(4.0, u1 / u2)
    assert m1 / m2 == m4
    assert m4.units == u1 / u2


def test_conditions():
    m1 = u.Measurement(1.0, "m")
    m2 = u.Measurement(3.0, u.Unit("error"))
    m3 = u.Measurement(2.4, "infinity")
    assert not m2.is_valid()
    assert m1.is_normal()
    assert not m2.is_normal()

    assert m1.is_valid()

    assert not m3.is_normal()
    assert m3.is_valid()


def test_root():
    m1 = u.Measurement("100 m^6 per second squared")
    m2 = m1.root(2)
    m3 = m1.sqrt()
    m4 = u.Measurement("10 m*m*m/s")
    assert m2 == m3
    assert m3 == m4


def test_pow():
    m1 = u.Measurement("10 m")
    m2 = m1**2
    m3 = m1**3
    assert m2 == m1 * m1
    assert m3 == m1 * m1 * m1


def test_comparisons():
    m1 = u.Measurement("10 m")
    m2 = u.Measurement("11 m")
    assert m1 < m2
    assert m2 > m1
    assert m1 >= m1
    assert m2 <= m2
    assert m2 >= m1
    assert m1 <= m2
    assert m1 != m2


def test_set_value():
    m1 = u.Measurement("100 months")
    assert m1.value == 100
    m2 = m1.set_value(14)
    assert m2.value == 14


def test_set_units():
    m1 = u.Measurement("100 months")
    assert m1.units == u.Unit("month")
    m2 = m1.set_units(u.Unit("day"))
    assert m2.units == u.Unit("day")


def test_value_as():
    m1 = u.Measurement("20 weeks")
    assert m1.value_as("day") == 20 * 7
    u1 = u.Unit("hr")
    assert m1.value_as(u1) == 20 * 7 * 24


def test_convert_to():
    m1 = u.Measurement("20 weeks")
    m2 = m1.convert_to("day")
    assert m2.value == 20 * 7
    u1 = u.Unit("hr")
    m3 = m1.convert_to("hr")
    assert m3.value == 20 * 7 * 24

    m2 = m1.to("day")
    assert m2.value == 20 * 7
    u1 = u.Unit("hr")
    m3 = m1.to("hr")
    assert m3.value == 20 * 7 * 24
    m4 = m1.convert_to_base()
    assert m4.units == u.Unit("s")
    assert m4.units.multiplier == 1.0
    assert m4.value == 20 * 7 * 24 * 3600


def test_as_unit():
    m1 = u.Measurement("15 seconds")
    m2 = u.Measurement(4, m1.as_unit())

    assert m2.value == 4
    assert m2.value_as("s") == 60
    assert float(m1) == 15
    assert m1


def test_add_sub():
    m1 = u.Measurement("15 seconds")
    m2 = u.Measurement(1, "minute")
    m3 = m2 - m1
    assert m3.value == 0.75

    m4 = m3 + m2 + m1
    assert m4 == u.Measurement(120, "second")


def test_negation():
    m1 = u.Measurement("15 seconds")
    m3 = -m1
    assert m3.value == -15.0


def test_pos():
    m1 = u.Measurement("15 seconds")
    m3 = +m1
    assert m1 == m3


def test_conditions():
    m1 = u.Measurement(34.5, "fq2te1tg1fe")
    assert not m1
    assert not bool(m1)

    m2 = u.Measurement(0, "m")
    assert not m2
    assert not bool(m2)
    assert m2.is_normal()


def test_mod():
    m1 = u.Measurement("18 seconds")
    m2 = u.Measurement("1 min")
    m3 = (m2 % m1).to("s")
    assert math.floor(m3.value) == 6
    m4 = m1 % 5
    assert m4.value == 3


def test_floor_div():
    m1 = u.Measurement("18 seconds")
    m2 = u.Measurement("1 min")
    m3 = m2 // m1
    assert m3.value == 3
    m4 = m1 // 4
    assert m4.value == 4
    assert m4.units == u.Unit("s")


def test_math_func():
    m1 = u.Measurement("15.78 seconds")
    m2 = u.Measurement("15.48 seconds")
    assert math.floor(m1).value == 15
    assert math.floor(m2).value == 15
    assert math.ceil(m1).value == 16
    assert math.ceil(m2).value == 16
    assert round(m1).value == 16
    assert round(m2).value == 15
    assert math.trunc(m1).value == 15
    assert math.trunc(m2).value == 15


def test_mult():
    m1 = u.Measurement("2 meters")
    m2 = u.Measurement(3, "meters")
    m3 = m2 * m1
    assert m3.value == 6

    m4 = 3 * m3
    assert m4 == u.Measurement(18, "meters squared")

    m5 = m3 * 3
    assert m5 == u.Measurement(18, "meters squared")


def test_vector_mult():
    m1 = u.Measurement("2 meters")
    m2 = u.Measurement(3, "meters")
    v1 = [5, 10, 15, 20, 25]
    mv3 = v1 * m1
    mv4 = m2 * v1

    assert mv3[2].value == 30
    assert mv4[3].value == 60


def test_div():
    m1 = u.Measurement("10 meters")
    m2 = u.Measurement(2, "seconds")
    m3 = m1 / m2
    assert m3.value == 5

    m4 = 10 / m3
    assert m4 == u.Measurement(2, "s/m")

    m5 = m3 / 2.5
    assert m5 == u.Measurement(2, "m/s")


def test_string():
    m1 = u.Measurement("10 lb")
    assert str(m1) == "10 lb"
    s3 = f"the measurement is {m1}"
    assert s3 == "the measurement is 10 lb"


def test_format():
    m1 = u.Measurement("9.7552 lb")
    s1 = f"the measurement is {m1:kg}"
    assert "kg" in s1

    s2 = f"the measurement is {m1:-}"
    assert s2 == "the measurement is 9.7552"

    s3 = f"the measurement is {m1:-kg}"
    assert "kg" not in s3

    m3 = u.Measurement("3.4")

    s4 = f"the measurement is {m3:-}"
    assert "3.4" in s4


def test_format_error():
    m1 = u.Measurement("9.7552 lb")
    try:
        s1 = f"the measurement is {m1:A}"
        assert False
    except ValueError:
        assert True

    try:
        s1 = f"the measurement is {m1:sgsg2362yqbvwsdddddqwadsa}"
        assert False
    except ValueError:
        assert True

    try:
        s1 = f"the measurement is {m1:-A}"
        assert False
    except ValueError:
        assert True

    try:
        s1 = f"the measurement is {m1:-sgsg2362yqbvwsdddddqwadsa}"
        assert False
    except ValueError:
        assert True


def test_close():
    m1 = u.Measurement("10 lb")
    m2 = u.Measurement("10.0000000001 lb")
    assert m1 != m2
    assert m1.isclose(m2)


def test_to_dict():
    m1 = u.Measurement(value=25, unit="lb")
    dv = m1.to_dict()
    assert dv["value"] == 25
    assert dv["unit"] == "lb"


def test_copy():
    m1 = u.Measurement("10 m")
    m2 = copy.copy(m1)
    assert m1 == m2


def test_deepcopy():
    m1 = u.Measurement("10 m")
    m2 = copy.deepcopy(m1)
    assert m1 == m2


def test_deepcopy_object():
    class val1:
        value1 = u.Measurement("10 m")
        value2 = u.Measurement("20 m")
        value3 = u.Measurement("30 m")
        values = [value1, value2, value3]

    v = val1()
    v2 = copy.deepcopy(v)
    assert v2.value1 == v.value1
    v.value1 = u.Measurement("50 m")
    assert v2.value1 == u.Measurement("10 m")



import pandas as pd

# Load the XLS file
file_path = "list-one.xls"
xls_data = pd.read_excel(file_path, sheet_name="Active")

outfile = open("currencydata.txt", "w")
for index, row in xls_data.iterrows():
    if index >= 3:
        currency = row.iloc[1]
        if currency in ["US Dollar", "Euro"]:
            continue
        if len(currency) > 39:
            continue
        currency = currency.replace(" ", "")
        line1 = (
            f'{{"{currency}",{{1.0, precise::currency, generateCurrencyCode("{row.iloc[2]}")}}}},'
        )
        outfile.write(f"{line1}\n")

outfile.write("\n\n\n")
outfile.write(
    "UNITS_CPP14_CONSTEXPR_OBJECT std::array<std::pair<const char*, std::uint32_t>, 1213> defined_commodity_codes{\n"
)

for index, row in xls_data.iterrows():
    if index >= 3:
        currency = row.iloc[1]
        if currency in ["US Dollar", "Euro"]:
            continue
        outfile.write(f'{{"{row.iloc[2]}",generateCurrencyCode("{row.iloc[2]}")}},\n')
        outfile.write(f'{{"{row.iloc[3]}",generateCurrencyCode("{row.iloc[2]}")}},\n')

outfile.write("};\n\n")



# Copyright (c) 2019-2025,
# Lawrence Livermore National Security, LLC;
# See the top-level NOTICE for additional details. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

import units_llnl as u


def test_convert_units():
    u1 = u.Unit("m")
    u2 = u.Unit("cm")
    v1 = u.convert(10, u1, u2)
    assert v1 == 10 * 100

    v2 = u.convert(unit_in=u1, unit_out=u2, value=20)
    assert v2 == 2000


def test_convert_string():
    v1 = u.convert(10, "m", "mm")
    assert v1 == 10 * 1000

    v2 = u.convert(unit_out="mm", unit_in="m", value=20)
    assert v2 == 20000


def test_convert_units_pu():
    u1 = u.Unit("puMW")
    u2 = u.Unit("kW")
    v1 = u.convert_pu(0.5, u1, u2, 100)
    assert v1 == 50000

    v2 = u.convert_pu(unit_in=u1, unit_out=u2, value=1.2, base=100)
    assert v2 == 120000


def test_convert_string_pu():
    v1 = u.convert_pu(0.5, "puMW", "kW", 100)
    assert v1 == 50000

    v2 = u.convert_pu(unit_in="puMW", unit_out="kW", value=1.2, base=100)
    assert v2 == 120000



# Copyright (c) 2019-2025,
# Lawrence Livermore National Security, LLC;
# See the top-level NOTICE for additional details. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

from units_llnl import Unit, add_user_defined_unit, defined_units_from_file


def test_user_defined_unit():
    clucks = Unit("19.3 m*A")

    add_user_defined_unit("clucks", clucks)
    add_user_defined_unit("sclucks", "23 m*mol*$")

    assert Unit("clucks/A") == Unit("19.3 m")
    assert Unit("sclucks/$") == Unit("23 m*mol")

    assert str(clucks) == "clucks"
    assert str(~clucks) == "1/clucks"

    sclucks = Unit("sclucks")
    assert not sclucks.is_error()
    assert str(Unit("ug") / sclucks**3) == "ug/sclucks^3"


def test_user_defined_unit_from_file():
    import os.path

    script_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    outputstr = defined_units_from_file(
        os.path.join(script_dir, "files/test_unit_files/other_units2.txt")
    )
    assert not outputstr
    y1 = Unit("yodles")
    assert y1 == Unit("73.0 count")

    y2 = Unit("yeedles")
    assert y2 == Unit("19.0 yodles")
    y3 = Unit("yimdles")
    assert y3 == Unit("12 yeedles")
    assert y3 == Unit("19*yodles*12")



# Copyright (c) 2019-2025,
# Lawrence Livermore National Security, LLC;
# See the top-level NOTICE for additional details. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

import units_llnl as u

from units_llnl import Unit, Measurement, Dimension, asdimension

import copy


def test_dimensions():
    d1 = Dimension("length")
    d2 = Dimension("distance")
    assert d1 == d2


def test_dimensions_with_bracketstring():
    d1 = Dimension("[length]")
    d2 = Dimension("[distance]")
    assert d1 == d2


def test_dimension_unit_constructor():
    d1 = Dimension(Unit("m"))
    d2 = Dimension(Unit("ft"))
    d3 = Dimension("[length]")
    assert d1 == d2
    assert d2 == d3


def test_dimension_unit_dim_prop():
    d1 = Unit("m").dimension
    d2 = Unit("ft").dimension
    d3 = Dimension("[length]")
    assert d1 == d2
    assert d2 == d3


def test_dimension_measure_constructor():
    d1 = Dimension(Measurement("10 m"))
    d2 = Dimension(Measurement("0.2586 ft"))
    d3 = Dimension("length")
    assert d1 == d2
    assert d3 == d2


def test_dimension_measure_dim_prop():
    d1 = Measurement("10 m").dimension
    d2 = Measurement("0.2586 ft").dimension
    d3 = Dimension("length")
    assert d1 == d2
    assert d3 == d2


def test_dimensions_multiply():
    d1 = Dimension("length")
    d2 = Dimension("distance")
    d3 = d1 * d2
    assert str(d3) == "[area]"


def test_dimensions_divide():
    d1 = Dimension("volume")
    d2 = Dimension("area")
    d3 = d1 / d2
    assert str(d3) == "[length]"


def test_dimensions_units():
    d1 = Dimension("volume")
    u1 = Unit("m^3")
    assert d1.default_unit == u1


def test_dimensions_decompose1():
    d1 = Dimension("volume")
    decomposition = d1.decompose()

    assert decomposition["Length"] == 3


def test_dimensions_decompose2():
    u1 = Unit("$ per watt radian")
    dim = u1.dimension
    decomposition = dim.decompose()
    print(f"dim={decomposition}")
    dim2 = Dimension(decomposition)
    print(f"dim2={dim2.decompose()}")
    assert dim2 == dim
    assert dim2.default_unit == u1


def test_copy():
    m1 = u.Dimension("10 m")
    m2 = copy.copy(m1)
    assert m1 == m2


def test_deepcopy():
    m1 = u.Dimension("10 m")
    m2 = copy.deepcopy(m1)
    assert m1 == m2


def test_deepcopy_object():
    class val1:
        value1 = u.Dimension("10 m")
        value2 = u.Dimension("20 kg")
        value3 = u.Dimension("30 A")
        values = [value1, value2, value3]

    v = val1()
    v2 = copy.deepcopy(v)
    assert v2.value1 == v.value1
    v.value1 = u.Dimension("50 mol")
    assert v2.value1 == u.Dimension("10 m")



from typing import TypeAlias
from .units_llnl_ext import (
    Unit,
    Measurement,
    Dimension,
    convert,
    convert_pu,
    default_unit,
    add_user_defined_unit,
    defined_units_from_file,
    __doc__,
)

Quantity: TypeAlias = Measurement


def asdimension(arg) -> Dimension:
    return Dimension(arg)


def asunit(*args) -> Unit:
    return Unit(*args)


def asquantity(*args) -> Quantity:
    return Measurement(*args)



# Copyright (c) 2019-2025,
# Lawrence Livermore National Security, LLC;
# See the top-level NOTICE for additional details. All rights reserved.
# SPDX-License-Identifier: BSD-3-Clause

import units_llnl as u
import copy


def test_basic_unit():
    u1 = u.Unit("m")
    u2 = u.Unit("s")
    u3 = u1 / u2
    u4 = u.Unit("mph")
    assert u3.is_convertible_to(u4)


def test_unit_constructor():
    u1 = u.Unit("cm")
    u2 = u.Unit(100, u1)

    assert u2 == u.Unit("m")
    assert u2


def test_unit_constructor2():
    u1 = u.Unit(unit="cm")
    u2 = u.Unit(100, u1)

    assert u2 == u.Unit(unit="m")
    assert u2


def test_unit_constructor3():
    u1 = u.Unit(unit="kg", multiplier=10, commodity="gold")

    assert u1.multiplier == 10.0
    assert u1.commodity == "gold"
    assert u1.base_units == u.Unit("kg")


def test_basic_multiplication():
    u1 = u.Unit("m")
    u2 = u.Unit("s")
    u3 = u1 / (u2 * u2)
    assert u3 == u.Unit("m/s^2")


def test_conditions():
    u1 = u.Unit("m")
    u2 = u.Unit("error")
    u3 = u.Unit("infinity")
    assert not u1.is_error()
    assert bool(u1)
    assert u2.is_error()
    assert u1.is_normal()
    assert not u2.is_normal()
    assert not bool(u2)
    assert u1.is_valid()

    assert u1.isfinite()
    assert not u1.isinf()
    assert u3.is_valid()
    assert not u3.isfinite()
    assert u3.isinf()

    u4 = u.Unit("puMW")
    assert u4.is_per_unit()
    assert not u3.is_per_unit()


def test_bad_unit():
    ue = u.Unit("qdfgqtegqgqg")
    assert not ue
    assert ue.is_error()


def test_zero():
    u1 = u.Unit("0")
    u2 = u.Unit("nan")
    u3 = u.Unit("m/s")
    u4 = u.Unit(0, u3)
    assert not bool(u1)
    assert not u1
    assert not bool(u2)
    assert not u2
    assert u3
    assert bool(u3)
    assert not bool(u4)
    assert not u4


def test_root():
    u1 = u.Unit("m^6 per second squared")
    u2 = u1.root(2)
    u3 = u1.sqrt()
    u4 = u.Unit("m*m*m/s")
    assert u2 == u3
    assert u3 == u4


def test_base():
    u1 = u.Unit("3 ft")
    u2 = u.Unit("yd")
    u3 = u.Unit("mm")
    assert u1.is_exactly_the_same(u1)
    assert u1 == u2
    assert u1.has_same_base(u2)
    assert u2.has_same_base(u3)


def test_multiplier():
    u1 = u.Unit("nm")
    assert u1.multiplier == 1e-9
    u2 = u1.set_multiplier(1e-6)
    assert u2 == u.Unit("um")
    assert u2.multiplier == 1e-6
    assert u1 != u2


def test_commodity():
    u1 = u.Unit("lb", "gold")
    assert u1.commodity == "gold"
    u2 = u1.set_commodity("silver")
    assert u2.commodity == "silver"


def test_string():
    u1 = u.Unit("lb")
    assert str(u1) == "lb"
    s3 = f"the unit is {u1}"
    assert s3 == "the unit is lb"
    u3 = u.Unit()
    assert str(u3) == ""


def test_inv():
    u1 = u.Unit("s")
    assert ~u1 == u.Unit("Hz")
    u3 = ~(~u1)
    assert u3 == u1


def test_hash():
    u1 = u.Unit("25.6 in")
    u2 = u1
    u3 = u.Unit("11.3 m")
    h1 = hash(u1)
    h2 = hash(u2)
    h3 = hash(u3)
    assert h1 == h2
    assert h1 != h3


def test_dictionary():
    d1 = {}

    u1 = u.Unit("25.6 in")
    u2 = u1
    u3 = u.Unit("11.3 m")
    d1[u1] = "in"
    d1[u3] = "m"

    assert d1[u2] == "in"
    assert d1[u3] == "m"


def test_to_dict():
    u1 = u.Unit(10.0, "m")
    v1 = u1.to_dict()
    assert v1["unit"] == "10m"


def test_float_mult():
    u1 = u.Unit("m")
    m3 = 10 * u1
    assert type(m3).__name__ == "Measurement"
    assert m3.value == 10

    m4 = u1 * 12
    assert type(m4).__name__ == "Measurement"
    assert m4.value == 12


def test_float_rlshift():
    u1 = u.Unit("m")
    m3 = 10 << u1
    assert type(m3).__name__ == "Measurement"
    assert m3.value == 10

    m4 = 12 << u1
    assert type(m4).__name__ == "Measurement"
    assert m4.value == 12


def test_mult_vect():
    u1 = u.Unit("m")

    v1 = [10, 20, 40]
    mv = v1 * u1
    assert type(mv[0]).__name__ == "Measurement"
    print(mv)
    assert mv[1].value == 20

    mv = u1 * v1
    assert type(mv[2]).__name__ == "Measurement"
    print(mv)
    assert mv[2].value == 40


def test_rlshift_vect():
    u1 = u.Unit("m")

    v1 = [10, 20, 40]
    mv = v1 << u1
    assert type(mv[0]).__name__ == "Measurement"
    print(mv)
    assert mv[1].value == 20


def test_convert_units():
    u1 = u.Unit("m")
    u2 = u.Unit("cm")
    v1 = u1.convert(10, u2)
    assert v1 == 10 * 100

    v2 = u1.convert(unit_out=u2, value=20)
    assert v2 == 2000

    v1 = u1.to(10, u2)
    assert v1 == 10 * 100

    v2 = u1.to(unit_out=u2, value=20)
    assert v2 == 2000


def test_convert_string():
    u1 = u.Unit("m")
    v1 = u1.convert(10, "mm")
    assert v1 == 10 * 1000

    v2 = u1.convert(unit_out="mm", value=20)
    assert v2 == 20000

    v1 = u1.to(10, "mm")
    assert v1 == 10 * 1000

    v2 = u1.to(unit_out="mm", value=20)
    assert v2 == 20000


def test_copy():
    u1 = u.Unit("10 m")
    u2 = copy.copy(u1)
    assert u1 == u2


def test_deepcopy():
    u1 = u.Unit("10 m")
    u2 = copy.deepcopy(u1)
    assert u1 == u2


def test_deepcopy_object():
    class val1:
        value1 = u.Unit("10 m")
        value2 = u.Unit("20 m")
        value3 = u.Unit("30 m")
        values = [value1, value2, value3]

    v = val1()
    v2 = copy.deepcopy(v)
    assert v2.value1 == v.value1
    v.value1 = u.Unit("50 m")
    assert v2.value1 == u.Unit("10 m")
